/* automatically generated from xfce4-session-settings.glade */
#ifdef __SUNPRO_C
#pragma align 4 (xfce4_session_settings_ui)
#endif
#ifdef __GNUC__
static const char xfce4_session_settings_ui[] __attribute__ ((__aligned__ (4))) =
#else
static const char xfce4_session_settings_ui[] =
#endif
{
  "<?xml version=\"1.0\"?><interface><object class=\"XfceTitledDialog\" id"
  "=\"xfce4_session_settings_dialog\"><property name=\"title\" translatabl"
  "e=\"yes\">Session and Startup</property><property name=\"window_positio"
  "n\">center-on-parent</property><property name=\"icon_name\">xfce4-sessi"
  "on</property><property name=\"type_hint\">dialog</property><property na"
  "me=\"has_separator\">False</property><property name=\"subtitle\" transl"
  "atable=\"yes\">Customize desktop startup and splash screen</property><c"
  "hild internal-child=\"vbox\"><object class=\"GtkVBox\" id=\"dialog-vbox"
  "1\"><property name=\"visible\">True</property><property name=\"orientat"
  "ion\">vertical</property><property name=\"spacing\">2</property><child>"
  "<object class=\"GtkNotebook\" id=\"plug-child\"><property name=\"visibl"
  "e\">True</property><property name=\"can_focus\">True</property><propert"
  "y name=\"border_width\">6</property><child><object class=\"GtkVBox\" id"
  "=\"vbox1\"><property name=\"visible\">True</property><property name=\"b"
  "order_width\">12</property><property name=\"orientation\">vertical</pro"
  "perty><property name=\"spacing\">6</property><child><object class=\"Gtk"
  "Frame\" id=\"frame1\"><property name=\"visible\">True</property><proper"
  "ty name=\"label_xalign\">0</property><property name=\"shadow_type\">non"
  "e</property><child><object class=\"GtkAlignment\" id=\"alignment2\"><pr"
  "operty name=\"visible\">True</property><property name=\"top_padding\">6"
  "</property><property name=\"bottom_padding\">6</property><property name"
  "=\"left_padding\">12</property><child><object class=\"GtkCheckButton\" "
  "id=\"chk_display_chooser\"><property name=\"label\" translatable=\"yes\""
  ">_Display chooser on login</property><property name=\"visible\">True</p"
  "roperty><property name=\"can_focus\">True</property><property name=\"re"
  "ceives_default\">False</property><property name=\"tooltip_text\" transl"
  "atable=\"yes\">Display the session chooser every time Xfce starts</prop"
  "erty><property name=\"use_underline\">True</property><property name=\"d"
  "raw_indicator\">True</property></object></child></object></child><child"
  " type=\"label\"><object class=\"GtkLabel\" id=\"label6\"><property name"
  "=\"visible\">True</property><property name=\"label\" translatable=\"yes"
  "\">&lt;b&gt;Session Chooser&lt;/b&gt;</property><property name=\"use_ma"
  "rkup\">True</property></object></child></object><packing><property name"
  "=\"expand\">False</property><property name=\"position\">0</property></p"
  "acking></child><child><object class=\"GtkFrame\" id=\"frame2\"><propert"
  "y name=\"visible\">True</property><property name=\"label_xalign\">0</pr"
  "operty><property name=\"shadow_type\">none</property><child><object cla"
  "ss=\"GtkAlignment\" id=\"alignment3\"><property name=\"visible\">True</"
  "property><property name=\"top_padding\">6</property><property name=\"bo"
  "ttom_padding\">6</property><property name=\"left_padding\">12</property"
  "><child><object class=\"GtkVBox\" id=\"vbox2\"><property name=\"visible"
  "\">True</property><property name=\"orientation\">vertical</property><pr"
  "operty name=\"spacing\">6</property><child><object class=\"GtkCheckButt"
  "on\" id=\"chk_session_autosave\"><property name=\"label\" translatable="
  "\"yes\">Automatically save session on _logout</property><property name="
  "\"visible\">True</property><property name=\"can_focus\">True</property>"
  "<property name=\"receives_default\">False</property><property name=\"to"
  "oltip_text\" translatable=\"yes\">Always save the session when logging "
  "out</property><property name=\"use_underline\">True</property><property"
  " name=\"draw_indicator\">True</property></object><packing><property nam"
  "e=\"expand\">False</property><property name=\"position\">0</property></"
  "packing></child><child><object class=\"GtkCheckButton\" id=\"chk_logout"
  "_prompt\"><property name=\"label\" translatable=\"yes\">_Prompt on logo"
  "ut</property><property name=\"visible\">True</property><property name=\""
  "can_focus\">True</property><property name=\"receives_default\">False</p"
  "roperty><property name=\"tooltip_text\" translatable=\"yes\">Prompt for"
  " confirmation when logging out</property><property name=\"use_underline"
  "\">True</property><property name=\"active\">True</property><property na"
  "me=\"draw_indicator\">True</property></object><packing><property name=\""
  "expand\">False</property><property name=\"position\">1</property></pack"
  "ing></child></object></child></object></child><child type=\"label\"><ob"
  "ject class=\"GtkLabel\" id=\"label7\"><property name=\"visible\">True</"
  "property><property name=\"label\" translatable=\"yes\">&lt;b&gt;Logout "
  "Settings&lt;/b&gt;</property><property name=\"use_markup\">True</proper"
  "ty></object></child></object><packing><property name=\"position\">1</pr"
  "operty></packing></child></object></child><child type=\"tab\"><object c"
  "lass=\"GtkLabel\" id=\"label1\"><property name=\"visible\">True</proper"
  "ty><property name=\"label\" translatable=\"yes\">_General</property><pr"
  "operty name=\"use_underline\">True</property></object><packing><propert"
  "y name=\"tab_fill\">False</property></packing></child><child><object cl"
  "ass=\"GtkHBox\" id=\"hbox2\"><property name=\"visible\">True</property>"
  "<property name=\"border_width\">12</property><property name=\"spacing\""
  ">12</property><child><object class=\"GtkVBox\" id=\"vbox6\"><property n"
  "ame=\"visible\">True</property><property name=\"orientation\">vertical<"
  "/property><property name=\"spacing\">6</property><child><object class=\""
  "GtkScrolledWindow\" id=\"scrolledwindow2\"><property name=\"visible\">T"
  "rue</property><property name=\"can_focus\">True</property><property nam"
  "e=\"hscrollbar_policy\">automatic</property><property name=\"vscrollbar"
  "_policy\">automatic</property><property name=\"shadow_type\">etched-in<"
  "/property><child><object class=\"GtkTreeView\" id=\"treeview_splash\"><"
  "property name=\"visible\">True</property><property name=\"can_focus\">T"
  "rue</property><property name=\"headers_visible\">False</property></obje"
  "ct></child></object><packing><property name=\"position\">0</property></"
  "packing></child><child><object class=\"GtkButton\" id=\"btn_splash_conf"
  "igure\"><property name=\"visible\">True</property><property name=\"sens"
  "itive\">False</property><property name=\"can_focus\">True</property><pr"
  "operty name=\"receives_default\">True</property><property name=\"toolti"
  "p_text\" translatable=\"yes\">Opens the configuration panel for the sel"
  "ected splash screen</property><child><object class=\"GtkHBox\" id=\"hbo"
  "x3\"><property name=\"visible\">True</property><property name=\"spacing"
  "\">4</property><child><object class=\"GtkImage\" id=\"image2\"><propert"
  "y name=\"visible\">True</property><property name=\"xalign\">1</property"
  "><property name=\"stock\">gtk-preferences</property></object><packing><"
  "property name=\"position\">0</property></packing></child><child><object"
  " class=\"GtkLabel\" id=\"label11\"><property name=\"visible\">True</pro"
  "perty><property name=\"xalign\">0</property><property name=\"label\" tr"
  "anslatable=\"yes\">Con_figure</property><property name=\"use_underline\""
  ">True</property><property name=\"mnemonic_widget\">btn_splash_configure"
  "</property></object><packing><property name=\"position\">1</property></"
  "packing></child></object></child></object><packing><property name=\"exp"
  "and\">False</property><property name=\"position\">1</property></packing"
  "></child><child><object class=\"GtkButton\" id=\"btn_splash_test\"><pro"
  "perty name=\"visible\">True</property><property name=\"sensitive\">Fals"
  "e</property><property name=\"can_focus\">True</property><property name="
  "\"receives_default\">True</property><property name=\"tooltip_text\" tra"
  "nslatable=\"yes\">Demonstrates the selected splash screen</property><ch"
  "ild><object class=\"GtkHBox\" id=\"hbox4\"><property name=\"visible\">T"
  "rue</property><property name=\"spacing\">4</property><child><object cla"
  "ss=\"GtkImage\" id=\"image3\"><property name=\"visible\">True</property"
  "><property name=\"xalign\">1</property><property name=\"stock\">gtk-exe"
  "cute</property></object><packing><property name=\"position\">0</propert"
  "y></packing></child><child><object class=\"GtkLabel\" id=\"label12\"><p"
  "roperty name=\"visible\">True</property><property name=\"xalign\">0</pr"
  "operty><property name=\"label\" translatable=\"yes\">_Test</property><p"
  "roperty name=\"use_underline\">True</property><property name=\"mnemonic"
  "_widget\">btn_splash_test</property></object><packing><property name=\""
  "position\">1</property></packing></child></object></child></object><pac"
  "king><property name=\"expand\">False</property><property name=\"positio"
  "n\">2</property></packing></child></object><packing><property name=\"ex"
  "pand\">False</property><property name=\"position\">0</property></packin"
  "g></child><child><object class=\"GtkVBox\" id=\"vbox7\"><property name="
  "\"visible\">True</property><property name=\"orientation\">vertical</pro"
  "perty><property name=\"spacing\">6</property><child><object class=\"Gtk"
  "Image\" id=\"img_splash_preview\"><property name=\"visible\">True</prop"
  "erty><property name=\"stock\">gtk-missing-image</property></object><pac"
  "king><property name=\"position\">0</property></packing></child><child><"
  "object class=\"GtkFrame\" id=\"frame5\"><property name=\"visible\">True"
  "</property><property name=\"label_xalign\">0</property><child><object c"
  "lass=\"GtkAlignment\" id=\"alignment6\"><property name=\"visible\">True"
  "</property><property name=\"bottom_padding\">4</property><property name"
  "=\"left_padding\">12</property><child><object class=\"GtkTable\" id=\"t"
  "able1\"><property name=\"visible\">True</property><property name=\"n_ro"
  "ws\">4</property><property name=\"n_columns\">2</property><property nam"
  "e=\"column_spacing\">12</property><property name=\"row_spacing\">2</pro"
  "perty><child><object class=\"GtkLabel\" id=\"lbl_splash_desc0\"><proper"
  "ty name=\"visible\">True</property><property name=\"xalign\">0</propert"
  "y><property name=\"label\" translatable=\"yes\">&lt;b&gt;Description:&l"
  "t;/b&gt;</property><property name=\"use_markup\">True</property></objec"
  "t><packing><property name=\"x_options\">GTK_FILL</property></packing></"
  "child><child><object class=\"GtkLabel\" id=\"lbl_splash_version0\"><pro"
  "perty name=\"visible\">True</property><property name=\"xalign\">0</prop"
  "erty><property name=\"label\" translatable=\"yes\">&lt;b&gt;Version:&lt"
  ";/b&gt;</property><property name=\"use_markup\">True</property></object"
  "><packing><property name=\"top_attach\">1</property><property name=\"bo"
  "ttom_attach\">2</property><property name=\"x_options\">GTK_FILL</proper"
  "ty></packing></child><child><object class=\"GtkLabel\" id=\"lbl_splash_"
  "author0\"><property name=\"visible\">True</property><property name=\"xa"
  "lign\">0</property><property name=\"label\" translatable=\"yes\">&lt;b&"
  "gt;Author:&lt;/b&gt;</property><property name=\"use_markup\">True</prop"
  "erty></object><packing><property name=\"top_attach\">2</property><prope"
  "rty name=\"bottom_attach\">3</property><property name=\"x_options\">GTK"
  "_FILL</property></packing></child><child><object class=\"GtkLabel\" id="
  "\"lbl_splash_homepage0\"><property name=\"visible\">True</property><pro"
  "perty name=\"xalign\">0</property><property name=\"label\" translatable"
  "=\"yes\">&lt;b&gt;Homepage:&lt;/b&gt;</property><property name=\"use_ma"
  "rkup\">True</property></object><packing><property name=\"top_attach\">3"
  "</property><property name=\"bottom_attach\">4</property><property name="
  "\"x_options\">GTK_FILL</property></packing></child><child><object class"
  "=\"GtkLabel\" id=\"lbl_splash_desc1\"><property name=\"visible\">True</"
  "property><property name=\"xalign\">0</property><property name=\"label\""
  " translatable=\"yes\">label</property><property name=\"selectable\">Tru"
  "e</property></object><packing><property name=\"left_attach\">1</propert"
  "y><property name=\"right_attach\">2</property></packing></child><child>"
  "<object class=\"GtkLabel\" id=\"lbl_splash_version1\"><property name=\""
  "visible\">True</property><property name=\"xalign\">0</property><propert"
  "y name=\"label\" translatable=\"yes\">label</property><property name=\""
  "selectable\">True</property></object><packing><property name=\"left_att"
  "ach\">1</property><property name=\"right_attach\">2</property><property"
  " name=\"top_attach\">1</property><property name=\"bottom_attach\">2</pr"
  "operty></packing></child><child><object class=\"GtkLabel\" id=\"lbl_spl"
  "ash_author1\"><property name=\"visible\">True</property><property name="
  "\"xalign\">0</property><property name=\"label\" translatable=\"yes\">la"
  "bel</property><property name=\"selectable\">True</property></object><pa"
  "cking><property name=\"left_attach\">1</property><property name=\"right"
  "_attach\">2</property><property name=\"top_attach\">2</property><proper"
  "ty name=\"bottom_attach\">3</property></packing></child><child><object "
  "class=\"GtkLabel\" id=\"lbl_splash_homepage1\"><property name=\"visible"
  "\">True</property><property name=\"xalign\">0</property><property name="
  "\"label\" translatable=\"yes\">label</property><property name=\"selecta"
  "ble\">True</property></object><packing><property name=\"left_attach\">1"
  "</property><property name=\"right_attach\">2</property><property name=\""
  "top_attach\">3</property><property name=\"bottom_attach\">4</property><"
  "/packing></child></object></child></object></child><child type=\"label\""
  "><object class=\"GtkLabel\" id=\"label13\"><property name=\"visible\">T"
  "rue</property><property name=\"label\" translatable=\"yes\">&lt;b&gt;In"
  "formation&lt;/b&gt;</property><property name=\"use_markup\">True</prope"
  "rty></object></child></object><packing><property name=\"expand\">False<"
  "/property><property name=\"position\">1</property></packing></child></o"
  "bject><packing><property name=\"position\">1</property></packing></chil"
  "d></object><packing><property name=\"position\">1</property></packing><"
  "/child><child type=\"tab\"><object class=\"GtkLabel\" id=\"label2\"><pr"
  "operty name=\"visible\">True</property><property name=\"label\" transla"
  "table=\"yes\">S_plash</property><property name=\"use_underline\">True</"
  "property></object><packing><property name=\"position\">1</property><pro"
  "perty name=\"tab_fill\">False</property></packing></child><child><objec"
  "t class=\"GtkVBox\" id=\"vbox8\"><property name=\"visible\">True</prope"
  "rty><property name=\"border_width\">12</property><property name=\"orien"
  "tation\">vertical</property><property name=\"spacing\">6</property><chi"
  "ld><object class=\"GtkHBox\" id=\"hbox10\"><property name=\"visible\">T"
  "rue</property><property name=\"spacing\">12</property><child><object cl"
  "ass=\"GtkImage\" id=\"image10\"><property name=\"visible\">True</proper"
  "ty><property name=\"stock\">gtk-dialog-info</property><property name=\""
  "icon-size\">6</property></object><packing><property name=\"expand\">Fal"
  "se</property><property name=\"position\">0</property></packing></child>"
  "<child><object class=\"GtkLabel\" id=\"label21\"><property name=\"visib"
  "le\">True</property><property name=\"xalign\">0</property><property nam"
  "e=\"label\" translatable=\"yes\">These applications are a part of the c"
  "urrently-running session, and can be saved when you log out.  Changes b"
  "elow will only take effect when the session is saved.</property><proper"
  "ty name=\"justify\">fill</property><property name=\"wrap\">True</proper"
  "ty></object><packing><property name=\"expand\">False</property><propert"
  "y name=\"position\">1</property></packing></child></object><packing><pr"
  "operty name=\"expand\">False</property><property name=\"position\">0</p"
  "roperty></packing></child><child><object class=\"GtkScrolledWindow\" id"
  "=\"scrolledwindow4\"><property name=\"visible\">True</property><propert"
  "y name=\"can_focus\">True</property><property name=\"hscrollbar_policy\""
  ">automatic</property><property name=\"vscrollbar_policy\">automatic</pr"
  "operty><property name=\"shadow_type\">etched-in</property><child><objec"
  "t class=\"GtkTreeView\" id=\"treeview_clients\"><property name=\"visibl"
  "e\">True</property><property name=\"can_focus\">True</property></object"
  "></child></object><packing><property name=\"position\">1</property></pa"
  "cking></child><child><object class=\"GtkHBox\" id=\"hbox7\"><property n"
  "ame=\"visible\">True</property><property name=\"spacing\">6</property><"
  "child><object class=\"GtkButton\" id=\"btn_save_session\"><property nam"
  "e=\"visible\">True</property><property name=\"can_focus\">True</propert"
  "y><property name=\"receives_default\">True</property><child><object cla"
  "ss=\"GtkHBox\" id=\"hbox5\"><property name=\"visible\">True</property><"
  "property name=\"spacing\">4</property><child><object class=\"GtkImage\""
  " id=\"image4\"><property name=\"visible\">True</property><property name"
  "=\"xalign\">1</property><property name=\"stock\">gtk-save</property></o"
  "bject><packing><property name=\"position\">0</property></packing></chil"
  "d><child><object class=\"GtkLabel\" id=\"label5\"><property name=\"visi"
  "ble\">True</property><property name=\"xalign\">0</property><property na"
  "me=\"label\" translatable=\"yes\">Save Sessio_n</property><property nam"
  "e=\"use_underline\">True</property></object><packing><property name=\"p"
  "osition\">1</property></packing></child></object></child></object><pack"
  "ing><property name=\"expand\">False</property><property name=\"position"
  "\">1</property></packing></child><child><object class=\"GtkButton\" id="
  "\"btn_clear_sessions\"><property name=\"visible\">True</property><prope"
  "rty name=\"sensitive\">True</property><property name=\"can_focus\">True"
  "</property><property name=\"receives_default\">True</property><property"
  " name=\"tooltip_text\" translatable=\"yes\">Empty the session cache</pr"
  "operty><child><object class=\"GtkHBox\" id=\"hbox9\"><property name=\"v"
  "isible\">True</property><property name=\"spacing\">4</property><child><"
  "object class=\"GtkImage\" id=\"image7\"><property name=\"visible\">True"
  "</property><property name=\"xalign\">1</property><property name=\"stock"
  "\">gtk-clear</property></object><packing><property name=\"position\">0<"
  "/property></packing></child><child><object class=\"GtkLabel\" id=\"labe"
  "l24\"><property name=\"visible\">True</property><property name=\"xalign"
  "\">0</property><property name=\"label\" translatable=\"yes\">_Clear sav"
  "ed sessions</property><property name=\"use_underline\">True</property><"
  "/object><packing><property name=\"position\">1</property></packing></ch"
  "ild></object></child></object><packing><property name=\"expand\">False<"
  "/property><property name=\"position\">1</property></packing></child><ch"
  "ild><object class=\"GtkButton\" id=\"btn_quit_client\"><property name=\""
  "visible\">True</property><property name=\"sensitive\">False</property><"
  "property name=\"can_focus\">True</property><property name=\"receives_de"
  "fault\">True</property><property name=\"tooltip_text\" translatable=\"y"
  "es\">Quit the program, and remove it from the session</property><child>"
  "<object class=\"GtkHBox\" id=\"hbox6\"><property name=\"visible\">True<"
  "/property><property name=\"spacing\">4</property><child><object class=\""
  "GtkImage\" id=\"image5\"><property name=\"visible\">True</property><pro"
  "perty name=\"xalign\">1</property><property name=\"stock\">gtk-quit</pr"
  "operty></object><packing><property name=\"position\">0</property></pack"
  "ing></child><child><object class=\"GtkLabel\" id=\"label23\"><property "
  "name=\"visible\">True</property><property name=\"xalign\">0</property><"
  "property name=\"label\" translatable=\"yes\">_Quit Program</property><p"
  "roperty name=\"use_underline\">True</property></object><packing><proper"
  "ty name=\"position\">1</property></packing></child></object></child></o"
  "bject><packing><property name=\"expand\">False</property><property name"
  "=\"pack_type\">end</property><property name=\"position\">1</property></"
  "packing></child></object><packing><property name=\"expand\">False</prop"
  "erty><property name=\"position\">2</property></packing></child></object"
  "><packing><property name=\"position\">2</property></packing></child><ch"
  "ild type=\"tab\"><object class=\"GtkLabel\" id=\"label3\"><property nam"
  "e=\"visible\">True</property><property name=\"label\" translatable=\"ye"
  "s\">_Session</property><property name=\"use_underline\">True</property>"
  "</object><packing><property name=\"position\">2</property><property nam"
  "e=\"tab_fill\">False</property></packing></child><child><object class=\""
  "GtkVBox\" id=\"vbox3\"><property name=\"visible\">True</property><prope"
  "rty name=\"border_width\">12</property><property name=\"orientation\">v"
  "ertical</property><property name=\"spacing\">6</property><child><object"
  " class=\"GtkFrame\" id=\"frame3\"><property name=\"visible\">True</prop"
  "erty><property name=\"label_xalign\">0</property><property name=\"shado"
  "w_type\">none</property><child><object class=\"GtkAlignment\" id=\"alig"
  "nment4\"><property name=\"visible\">True</property><property name=\"top"
  "_padding\">6</property><property name=\"bottom_padding\">6</property><p"
  "roperty name=\"left_padding\">12</property><child><object class=\"GtkVB"
  "ox\" id=\"vbox4\"><property name=\"visible\">True</property><property n"
  "ame=\"orientation\">vertical</property><property name=\"spacing\">6</pr"
  "operty><child><object class=\"GtkCheckButton\" id=\"chk_compat_gnome\">"
  "<property name=\"label\" translatable=\"yes\">Launch GN_OME services on"
  " startup</property><property name=\"visible\">True</property><property "
  "name=\"can_focus\">True</property><property name=\"receives_default\">F"
  "alse</property><property name=\"tooltip_text\" translatable=\"yes\">Sta"
  "rt GNOME services, such as gnome-keyring</property><property name=\"use"
  "_underline\">True</property><property name=\"draw_indicator\">True</pro"
  "perty></object><packing><property name=\"expand\">False</property><prop"
  "erty name=\"position\">0</property></packing></child><child><object cla"
  "ss=\"GtkCheckButton\" id=\"chk_compat_kde\"><property name=\"label\" tr"
  "anslatable=\"yes\">Launch _KDE services on startup</property><property "
  "name=\"visible\">True</property><property name=\"can_focus\">True</prop"
  "erty><property name=\"receives_default\">False</property><property name"
  "=\"tooltip_text\" translatable=\"yes\">Start KDE services, such as kdei"
  "nit</property><property name=\"use_underline\">True</property><property"
  " name=\"draw_indicator\">True</property></object><packing><property nam"
  "e=\"expand\">False</property><property name=\"position\">1</property></"
  "packing></child></object></child></object></child><child type=\"label\""
  "><object class=\"GtkLabel\" id=\"label8\"><property name=\"visible\">Tr"
  "ue</property><property name=\"label\" translatable=\"yes\">&lt;b&gt;Com"
  "patibility&lt;/b&gt;</property><property name=\"use_markup\">True</prop"
  "erty></object></child></object><packing><property name=\"expand\">False"
  "</property><property name=\"position\">0</property></packing></child><c"
  "hild><object class=\"GtkFrame\" id=\"frame4\"><property name=\"visible\""
  ">True</property><property name=\"label_xalign\">0</property><property n"
  "ame=\"shadow_type\">none</property><child><object class=\"GtkAlignment\""
  " id=\"alignment5\"><property name=\"visible\">True</property><property "
  "name=\"top_padding\">6</property><property name=\"bottom_padding\">6</p"
  "roperty><property name=\"left_padding\">12</property><child><object cla"
  "ss=\"GtkCheckButton\" id=\"chk_enable_tcp\"><property name=\"label\" tr"
  "anslatable=\"yes\">Manage _remote applications</property><property name"
  "=\"visible\">True</property><property name=\"can_focus\">True</property"
  "><property name=\"receives_default\">False</property><property name=\"t"
  "ooltip_text\" translatable=\"yes\">Manage remote applications over the "
  "network (this may be a security risk)</property><property name=\"use_un"
  "derline\">True</property><property name=\"draw_indicator\">True</proper"
  "ty></object></child></object></child><child type=\"label\"><object clas"
  "s=\"GtkLabel\" id=\"label9\"><property name=\"visible\">True</property>"
  "<property name=\"label\" translatable=\"yes\">&lt;b&gt;Security&lt;/b&g"
  "t;</property><property name=\"use_markup\">True</property></object></ch"
  "ild></object><packing><property name=\"expand\">False</property><proper"
  "ty name=\"position\">1</property></packing></child><child><object class"
  "=\"GtkFrame\" id=\"frame6\"><property name=\"visible\">True</property><"
  "property name=\"label_xalign\">0</property><property name=\"shadow_type"
  "\">none</property><child><object class=\"GtkAlignment\" id=\"alignment7"
  "\"><property name=\"visible\">True</property><property name=\"top_paddi"
  "ng\">6</property><property name=\"bottom_padding\">6</property><propert"
  "y name=\"left_padding\">12</property><child><object class=\"GtkCheckBut"
  "ton\" id=\"chk_lock_screen\"><property name=\"label\" translatable=\"ye"
  "s\">_Lock screen before sleep</property><property name=\"visible\">True"
  "</property><property name=\"can_focus\">True</property><property name=\""
  "receives_default\">False</property><property name=\"tooltip_text\" tran"
  "slatable=\"yes\">Run xflock4 before suspending or hibernating the syste"
  "m</property><property name=\"use_underline\">True</property><property n"
  "ame=\"draw_indicator\">True</property></object></child></object></child"
  "><child type=\"label\"><object class=\"GtkLabel\" id=\"label25\"><prope"
  "rty name=\"visible\">True</property><property name=\"label\" translatab"
  "le=\"yes\">&lt;b&gt;Shutdown&lt;/b&gt;</property><property name=\"use_m"
  "arkup\">True</property></object></child></object><packing><property nam"
  "e=\"expand\">False</property><property name=\"position\">2</property></"
  "packing></child></object><packing><property name=\"position\">3</proper"
  "ty></packing></child><child type=\"tab\"><object class=\"GtkLabel\" id="
  "\"label4\"><property name=\"visible\">True</property><property name=\"l"
  "abel\" translatable=\"yes\">Ad_vanced</property><property name=\"use_un"
  "derline\">True</property></object><packing><property name=\"position\">"
  "3</property><property name=\"tab_fill\">False</property></packing></chi"
  "ld></object><packing><property name=\"position\">1</property></packing>"
  "</child><child internal-child=\"action_area\"><object class=\"GtkHButto"
  "nBox\" id=\"dialog-action_area1\"><property name=\"visible\">True</prop"
  "erty><child><object class=\"GtkButton\" id=\"button1\"><property name=\""
  "label\">gtk-help</property><property name=\"visible\">True</property><p"
  "roperty name=\"can_focus\">True</property><property name=\"receives_def"
  "ault\">True</property><property name=\"use_stock\">True</property></obj"
  "ect><packing><property name=\"expand\">False</property><property name=\""
  "fill\">False</property><property name=\"position\">0</property></packin"
  "g></child><child><object class=\"GtkButton\" id=\"button2\"><property n"
  "ame=\"label\">gtk-close</property><property name=\"visible\">True</prop"
  "erty><property name=\"can_focus\">True</property><property name=\"recei"
  "ves_default\">True</property><property name=\"use_stock\">True</propert"
  "y></object><packing><property name=\"expand\">False</property><property"
  " name=\"fill\">False</property><property name=\"position\">1</property>"
  "</packing></child></object><packing><property name=\"expand\">False</pr"
  "operty><property name=\"pack_type\">end</property><property name=\"posi"
  "tion\">0</property></packing></child></object></child><action-widgets><"
  "action-widget response=\"-11\">button1</action-widget><action-widget re"
  "sponse=\"-7\">button2</action-widget></action-widgets></object><object "
  "class=\"GtkDialog\" id=\"dialog_saving\"><property name=\"border_width\""
  ">5</property><property name=\"title\" translatable=\"yes\">Saving Sessi"
  "on</property><property name=\"resizable\">False</property><property nam"
  "e=\"window_position\">center-on-parent</property><property name=\"destr"
  "oy_with_parent\">True</property><property name=\"icon_name\">xfce4-sess"
  "ion</property><property name=\"type_hint\">dialog</property><property n"
  "ame=\"skip_taskbar_hint\">True</property><property name=\"skip_pager_hi"
  "nt\">True</property><property name=\"transient_for\">xfce4_session_sett"
  "ings_dialog</property><property name=\"has_separator\">False</property>"
  "<child internal-child=\"vbox\"><object class=\"GtkVBox\" id=\"dialog-vb"
  "ox2\"><property name=\"visible\">True</property><property name=\"orient"
  "ation\">vertical</property><property name=\"spacing\">2</property><chil"
  "d><object class=\"GtkVBox\" id=\"vbox5\"><property name=\"visible\">Tru"
  "e</property><property name=\"border_width\">6</property><property name="
  "\"orientation\">vertical</property><property name=\"spacing\">6</proper"
  "ty><child><object class=\"GtkHBox\" id=\"hbox8\"><property name=\"visib"
  "le\">True</property><property name=\"spacing\">12</property><child><obj"
  "ect class=\"GtkImage\" id=\"image6\"><property name=\"visible\">True</p"
  "roperty><property name=\"stock\">gtk-dialog-info</property><property na"
  "me=\"icon-size\">6</property></object><packing><property name=\"expand\""
  ">False</property><property name=\"position\">0</property></packing></ch"
  "ild><child><object class=\"GtkLabel\" id=\"label10\"><property name=\"v"
  "isible\">True</property><property name=\"xalign\">0</property><property"
  " name=\"label\" translatable=\"yes\">Your session is being saved.  If y"
  "ou do not wish to wait, you may close this window.</property><property "
  "name=\"justify\">fill</property><property name=\"wrap\">True</property>"
  "</object><packing><property name=\"position\">1</property></packing></c"
  "hild></object><packing><property name=\"expand\">False</property><prope"
  "rty name=\"position\">0</property></packing></child><child><object clas"
  "s=\"GtkProgressBar\" id=\"progress_save_session\"><property name=\"visi"
  "ble\">True</property><property name=\"activity_mode\">True</property></"
  "object><packing><property name=\"expand\">False</property><property nam"
  "e=\"position\">1</property></packing></child></object><packing><propert"
  "y name=\"expand\">False</property><property name=\"position\">1</proper"
  "ty></packing></child><child internal-child=\"action_area\"><object clas"
  "s=\"GtkHButtonBox\" id=\"dialog-action_area2\"><property name=\"visible"
  "\">True</property><property name=\"layout_style\">end</property><child>"
  "<object class=\"GtkButton\" id=\"button3\"><property name=\"label\">gtk"
  "-close</property><property name=\"visible\">True</property><property na"
  "me=\"can_focus\">True</property><property name=\"receives_default\">Tru"
  "e</property><property name=\"use_stock\">True</property></object><packi"
  "ng><property name=\"expand\">False</property><property name=\"fill\">Fa"
  "lse</property><property name=\"position\">0</property></packing></child"
  "></object><packing><property name=\"expand\">False</property><property "
  "name=\"pack_type\">end</property><property name=\"position\">0</propert"
  "y></packing></child></object></child><action-widgets><action-widget res"
  "ponse=\"0\">button3</action-widget></action-widgets></object></interfac"
  "e>"
};

static const unsigned xfce4_session_settings_ui_length = 28653u;

